#include "Dasyne.hpp"
#include <iostream>

using namespace std;
using namespace libdasyne;

const uint16_t PORT_NUMBER = 5978;//The port number to bind to
static bool runProgram = true; //false if we should stop the server
static DEngine netEngine;//calls event handlers for networking classes
static DUdpSocket *udpSocket = new DUdpSocket(&netEngine);//sends and receives data

#ifdef DA_WINDOWS_VERSION
    #include "WindowsExamplesTools.hpp"
#endif

//_______________ Event Handlers _______________

//Event handler called when we receive data
void receiveHandler(DBuffer &receivedData,
                    string &sourceAddress,
                    uint16_t sourcePort)
{
    udpSocket->send(receivedData, sourceAddress, sourcePort);
}//receiveHandler

//Event handler called when a network error occurs.
void errorHandler(string message,
                  bool socketOK)
{
    if (!socketOK)
    {
        cout << "Socket Error: " << message << endl;
        runProgram = false;
    }//if !socketOK
    else
        cout << message << endl;
}//errorHandler

//_______________ Main Function _______________

int main(int argc,
         char *argv[])
{
    int eventsRun = 0;//the number of events dispatched by a call to update()

    //### Windows specific configuration - see WindowsExamplesTools.hpp ###
    #ifdef DA_WINDOWS_VERSION
        if (!initializeWindows())
            runProgram = false;
    #endif

    //### Bind the UDP socket ###
    if (runProgram)
    {
        udpSocket->setErrorHandler(&errorHandler);
        udpSocket->setReceiveHandler(&receiveHandler);
        if (udpSocket->bind(libdasyne::DA_IPV4, PORT_NUMBER))
        {
            cout << "Mirroring UDP packets back to sender, listening on port " << PORT_NUMBER << endl;
            cout << "Hit Ctrl-C to exit" << endl;
        }
        else
        {
            runProgram = false;
            cout << "Couldn't bind: " << udpSocket->getErrMsg() << endl;
        }
    }

    //### Periodically call event handlers ###
    while (runProgram)
    {
        eventsRun = netEngine.update();
        if (eventsRun < 0)
        {
            cout << netEngine.getErrMsg();
            runProgram = false;
        }

        //Sleep if we have nothing to do
        else if (eventsRun == 0)
            DEngine::sleep(1);
    }

    //### Delete the objects that use DEngine before it's deleted ###
    delete udpSocket;

    return 0;
}//main
