/* This file addresses two problems with running the example programs from Windows (MinGW-w64 build).

    1) Ctrl-C causes the example programs to freeze. We catch it here and manually tell the program
       to stop running.

    2) Windows does not schedule the programs to run frequently enough. We ask for 1 ms of sleep,
       but get up to 25 ms. This causes the latency and throughput tests to report extremely poor
       results. We manually adjust the scheduling priority here.
*/

#ifdef DA_WINDOWS_VERSION
    #include <Windows.h>
    #include <iostream>

    using namespace std;

    BOOL ControlHandler(DWORD fdwControlType)
    {
        switch (fdwControlType)
        {
            case CTRL_C_EVENT:
            case CTRL_CLOSE_EVENT:
            case CTRL_BREAK_EVENT:
                runProgram = false;
                return(TRUE);

            default:
                return FALSE;
        }
    }//ControlHandler

    bool initializeWindows(void)
    {
        bool result = true;//false on errors

        //### Listen for interrupt signals ###
        if (!SetConsoleCtrlHandler((PHANDLER_ROUTINE) ControlHandler, TRUE))
        {
            cout << "Couldn't install console ctrl handler!" << endl;
            result = false;
        }

        //### Ensure our event generator is scheduled to run at roughly 1 ms intervals ###
        if (result)
        {
            if (!SetPriorityClass(GetCurrentProcess(), ABOVE_NORMAL_PRIORITY_CLASS))
            {
                cout << "Couldn't set priority class to above normal" << endl;
                result = false;
            }
        }

        return result;
    }//initializeWindows
#endif
