/*
 * Copyright (C) 2011 - 2014 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

/* DESCRIPTION
   Iterates through the elements in a DBytes or DBuffer class, giving us a speedup
   of roughly 2 times compared to repeatedly calling getByte().
*/

#ifndef DBUFFERITER_HPP_INCLUDED
#define DBUFFERITER_HPP_INCLUDED

#include "DSafeArray.hpp"
#include "DEngine.hpp"

namespace libdasyne
{
    class DBytes;

    class DBufferIter : public DObject
    {
        public:
            //constructor
            DBufferIter(void);

            //copy constructor
            DBufferIter(const DBufferIter &source);

            //destructor
            ~DBufferIter(void);

            //Overloaded = operator
            DBufferIter& operator= (const DBufferIter &source);

            //Set the buffer to iterate through. The index is set to zero.
            void setBuffer(const DBytes *newBuffer);

            //Set the element in theBuffer that this iterator points to. Note: this function
            //is inefficient, and should not be used for iterating through the buffer.
            void setIndex(DA_UNSIGNED newIndex);

            //Returns a reference to the current byte in the buffer
            uint8_t& value(void) const;

            //Move to the next byte in theBuffer.
            void next(void);

            //Returns true if we've reached the end and value() should no longer be called.
            bool finished(void);

        private:
            //The current element in the DBytes that we're accessing (each element is an array)
            //Don't delete!
            #ifdef DA_DEBUG_MODE
                DSafeArray bytePointer;
            #else
                uint8_t* bytePointer;
            #endif

            //The buffer we're iterating through
            const DBytes *theBuffer;

            //The index of the current array in the DBytes
            DA_UNSIGNED dataIndex;

            //The largest array index we can access in the current DBytes element
            DA_UNSIGNED maxArrayIndex;

            //The index in the current array we're iterating through
            uint16_t arrayIndex;

            //True if we're out of bounds, and value() should not be called. Causes finished() to
            //return true.
            bool passedEnd;

            //############### FUNCTIONS ###############

            //Initialize member variables
            void initVars(void);

            //Copy the member variables from source
            void copyVars(const DBufferIter &source);

    };//DBufferIter
}//namespace libdasyne

#endif
