/*
 * Copyright (C) 2011-2015 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

#include "DEventMap.hpp"
#include "DBuffer.hpp"

using namespace std;
using namespace libdasyne;

DEventMap::DEventMap(uint8_t newMsgType)
{
    msgType = newMsgType;
    haveSingle = false;
}//constructor

DEventMap::~DEventMap(void)
{
}//destructor

void DEventMap::addHandler(boost::function<void (DBuffer&)> newHandler)
{
    if (handlers.size() == 0)
    {
        haveSingle = true;
        singleHandler = newHandler;
    }//if size()
    else
    {
        DEngine::addFatal("Tried to add a handler for message type ");
        DEngine::addFatal(DEngine::intToStr(msgType));
        DEngine::fatalError(" without giving a subtype, but there are handlers for subtypes defined");
    }//else size()
}//addHandler

void DEventMap::addHandler(uint16_t subType,
                           boost::function<void (DBuffer&)> newHandler)
{
    map<uint16_t, boost::function<void (DBuffer&)> >::iterator mapIter;//points to an element in handlers

    if (haveSingle)
    {
        DEngine::addFatal("Tried to add a handler for message type ");
        DEngine::addFatal(DEngine::intToStr(msgType));
        DEngine::addFatal(", subtype ");
        DEngine::addFatal(DEngine::intToStr(subType));
        DEngine::fatalError(", but it already has a handler with no subtype");
    }//if haveSingle

    mapIter = handlers.find(subType);
    if (mapIter == handlers.end())
        handlers.insert(pair<uint16_t, boost::function<void (DBuffer&)> >(subType, newHandler));
    else
        mapIter->second = newHandler;
}//addHandler

void DEventMap::removeHandler(void)
{
    if (!haveSingle)
        DEngine::fatalError("We don't have a single event handler for all sub-types.");

    haveSingle = false;

    if (handlers.size() != 0)
        DEngine::fatalError("Successfully removed a single handler, but the DEventMap has sub-handlers!?");
}//removeHandler

bool DEventMap::removeHandler(uint16_t subType)
{
    map<uint16_t, boost::function<void (DBuffer&)> >::iterator mapIter;//points to the element in handlers to remove
    bool result = false;//true if this was the last subtype handler

    //### Verify that we don't have a single handler ###
    if (haveSingle)
        DEngine::fatalError("Trying to remove a sub-handler, but this message type doesn't have subtypes");

    //### Find the handler to remove ###
    mapIter = handlers.find(subType);
    if (mapIter == handlers.end())
    {
        DEngine::addFatal("Tried to remove handler for subtype ");
        DEngine::addFatal(DEngine::intToStr(subType));
        DEngine::fatalError(", but there isn't one.");
    }//if mapIter

    //### Remove the handler ###
    handlers.erase(mapIter);
    if (handlers.size() == 0)
        result = true;

    return result;
}//removeHandler

bool DEventMap::getHandler(uint16_t subType,
                           boost::function<void (DBuffer&)> &theHandler)
{
    bool result = true;//false if we couldn't find a handler
    map<uint16_t, boost::function<void (DBuffer&)> >::iterator mapIter;//points to an element in handlers

    mapIter = handlers.find(subType);
    if (mapIter == handlers.end())
        result = false;
    else
        theHandler = mapIter->second;

    return result;
}//getHandler

unsigned int DEventMap::size(void)
{
    return handlers.size();
}//size()

bool DEventMap::getHaveSingle(void)
{
    return haveSingle;
}//getHaveSingle

boost::function<void (DBuffer&)> DEventMap::getSingleHandler(void)
{
    if (!haveSingle)
        DEngine::fatalError("DEventMap::getSingleHandler(): no single handler defined!");

    return singleHandler;
}//getSingleHandler
