/*
 * Copyright (C) 2011-2015 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

/* DESCRIPTION
    Contains an std::map that matches message sub-types to their function pointers.
    The appropriate function pointer will be called when a message of that sub-type
    is received.
*/

#ifndef DEVENTMAP_HPP_INCLUDED
#define DEVENTMAP_HPP_INCLUDED

#include "DEngine.hpp"

namespace libdasyne
{
    class DBuffer;

    class DEventMap : public DObject
    {
        public:
            //constructor
            DEventMap(uint8_t newMsgType);

            //destructor
            ~DEventMap(void);

            //Set the handler for the specified message type
            void addHandler(boost::function<void (DBuffer&)> newHandler);
            void addHandler(uint16_t subType,
                            boost::function<void (DBuffer&)> newHandler);

            //Remove the stored message handler, throws an exception if we aren't
            //using a single handler.
            void removeHandler(void);

            //Remove a handler for the specified message subtype. Returns true if there
            //are no more subtype handlers to remove.
            bool removeHandler(uint16_t subType);

            //Sets theHandler to the event handler for the specified subtype.
            //Returns false if no handler was found.
            bool getHandler(uint16_t subType,
                            boost::function<void (DBuffer&)> &theHandler);

            //Returns the event handler to use if there are no subtypes
            boost::function<void (DBuffer&)> getSingleHandler(void);

            //Returns true if there's no subtype for this message type
            bool getHaveSingle(void);

            //The number of subtype handlers in the map
            unsigned int size(void);

        private:
            //The key is a message sub-type, the value is an event handler to call
            std::map<uint16_t, boost::function<void (DBuffer&)> > handlers;

            //Function to call if there is no subtype for this message type
            boost::function<void (DBuffer&)> singleHandler;

            //The type of message we're parsing
            uint8_t msgType;

            //True if singleHandler points to a function, false otherwise
            bool haveSingle;

    };//DEventMap
}//namespace libdasyne

#endif
