/*
 * Copyright (C) 2005-2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

/* DESCRIPTION
  A cross-platform mutex. It's implemented with Pthreads and Windows native threads.
  We throw errors and set errMsg rather than use DEngine::fatalError, as fatalError
  needs to lock and unlock mutexes, which may in itself be the source of the error.
*/

#ifndef DMUTEX_HPP_INCLUDED
#define DMUTEX_HPP_INCLUDED

#include "DEngine.hpp"

namespace libdasyne
{
    class DMutex : public DObject
    {
        public:
            //constructor
            DMutex(void);

            //destructor
            ~DMutex(void);

            //restrict access to critical areas
            void lock(void);
            void unlock(void);

            //If one of the other member functions throws an exception, errMsg will be set
            //with a description of the problem
            std::string getErrMsg(void);

        private:
            //the mutex
            #ifdef DA_WINDOWS_VERSION
                HANDLE theMutex;
            #else
                pthread_mutex_t theMutex;
            #endif

            //A description of the most recent error
            std::string errMsg;
    };//DMutex
}//namespace libdasyne

#endif
