/* DESCRIPTION
    Boost sockets can produce events even after asynchronous operations are cancelled,
    such as by closing the socket. DSocket provides an interface that DEngine can use
    to safely delete these sockets, removing the burden of waiting until the socket can be
    deleted from the user.

    The DEngine will first close the socket, then wait for all pending events to be
    processed before deleting it. It must also have a way to disconnect user-specified handlers.

    Since the user has asked the DEngine to delete the socket, he will not expect his event handlers
    to be called after that point, and doing so may result in undefined behaviour if the event handlers
    belong to an object that the user has since deleted.
*/

#ifndef DSOCKET_HPP_INCLUDED
#define DSOCKET_HPP_INCLUDED

#include "DObject.hpp"

namespace libdasyne
{
    class DSocket : public DObject
    {
        public:
            //Destructor
            virtual ~DSocket(void)
            {
            }

            //Closes the socket.
            virtual bool closeSocket(void) = 0;

            //Removes all user-defined event handlers from this socket. This method should only
            //be called by the containing DEngine, since that's the only engine that knows when
            //pending events are complete.
            virtual void clearAllHandlers(void) = 0;
    };//DSocket
}//namespace libdasyne

#endif
